@@ -70,11 +70,13 @@ deleteEvents.redcapApiConnection <- function(rcon,
   
   if (response$status_code != 200) return(redcapError(response, error_handling))
   
-  message(sprintf("Events deleted: %s", 
-                  as.character(response)))
-  
   if (refresh && rcon$has_events()){
     rcon$refresh_events()
+    # changing events can change availability of arms
+    # and whether a project is considered longitudinal
+    rcon$refresh_arms()
     rcon$refresh_projectInformation()
   }
+  
+  invisible(as.character(response))
 }
