@@ -23,6 +23,7 @@ exportReportsTyped.redcapApiConnection <- function(rcon,
                                                    cast          = list(),
                                                    assignment    = list(label=stripHTMLandUnicode,
                                                                         units=unitsFieldAnnotation),
+                                                   warn_zero_coded = TRUE,
                                                    ..., 
                                                    config        = list(),
                                                    api_param     = list(),
@@ -108,10 +109,7 @@ exportReportsTyped.redcapApiConnection <- function(rcon,
                           body = c(body, api_param), 
                           config)
   
-  Raw <- utils::read.csv(text = as.character(response), 
-                         na.strings = "", 
-                         sep = csv_delimiter,
-                         stringsAsFactors = FALSE)
+  Raw <- as.data.frame(response, sep = csv_delimiter)
   
   if (length(drop_fields) > 0){
     Raw <- Raw[!names(Raw) %in% drop_fields]
@@ -129,5 +127,6 @@ exportReportsTyped.redcapApiConnection <- function(rcon,
                cast             = cast, 
                assignment       = assignment, 
                default_cast     = .default_cast, 
-               default_validate = .default_validate)
+               default_validate = .default_validate,
+               warn_zero_coded  = warn_zero_coded)
 }
