@@ -28,28 +28,25 @@ filterEmptyRow <- function(data,
   
   invalid <- attr(data, "invalid")
   
-  id_fields <- c(getProjectIdFields(rcon), 
-                 REDCAP_SYSTEM_FIELDS, 
+  id_fields <- c(getProjectIdFields(rcon), REDCAP_SYSTEM_FIELDS,
                  "redcap_data_access_group")
   
-  is_all_missing <- logical(nrow(data))
+  other_fields <- setdiff(names(data), id_fields)
   
-  for (i in seq_along(is_all_missing)){
-    this_row <- data[ i, names(data)[!names(data) %in% id_fields], drop = FALSE]
-    if (ncol(this_row) == 0){ # This occurs when all of the columns are ID fields.
-      is_all_missing[i] <- FALSE
-    } else if (all(is.na(unlist(this_row)))){
-      is_all_missing[i] <- TRUE
+  if(length(other_fields) == 0)
+    NewData <- data
+  
+  else {
+    is_all_missing <- apply(is.na(data[other_fields]), 1, all)
+    
+    NewData <- data[!is_all_missing, , drop = FALSE]
+    
+    for (field in names(NewData)) {
+      attributes(NewData[[field]]) <- attributes(data[[field]])
     }
+    
+    attr(NewData, "invalid") <- invalid
   }
   
-  NewData <- data[!is_all_missing, , drop = FALSE]
-  
-  for (field in names(NewData)){
-    attributes(NewData[[field]]) <- attributes(data[[field]])
-  }
-  
-  attr(NewData, "invalid") <- invalid
-  
   NewData
 }
