@@ -94,11 +94,13 @@ importArms.redcapApiConnection <- function(rcon,
   
   if (response$status_code != 200) return(redcapError(response, error_handling))
   
-  message(sprintf("Arms imported: %s", 
-                  as.character(response)))
-  
   if (refresh && rcon$has_arms()){
     rcon$refresh_arms()
+    # Changes to arms can impact events and if the project is 
+    # still considered longitudinal
+    rcon$refresh_events()
     rcon$refresh_projectInformation()
   }
+  
+  invisible(as.character(response))
 }
